C-----------------------------------------------------------------------
C     Copyright (C) 2003-2007, ENPC - INRIA - EDF R&D
C     Author(s): Debry Edouard
C     
C     This file is part of the Size Resolved Aerosol Model (SIREAM), a
C     component of the air quality modeling system Polyphemus.
C    
C     Polyphemus is developed in the INRIA - ENPC joint project-team
C     CLIME and in the ENPC - EDF R&D joint laboratory CEREA.
C    
C     Polyphemus is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published
C     by the Free Software Foundation; either version 2 of the License,
C     or (at your option) any later version.
C     
C     Polyphemus is distributed in the hope that it will be useful, but
C     WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
C     General Public License for more details.
C     
C     For more information, visit the Polyphemus web site:
C     http://cerea.enpc.fr/polyphemus/
C-----------------------------------------------------------------------

      SUBROUTINE aerosol (i1,i2,j1,j2,Nx,Ny,Nz,nesp,
     $     nesp_isorropia_loc,nesp_aec_loc,nesp_pankow_loc,
     $     LWCmin,ts,DLhumid,
     $     DLtemp,DLpress,delta_t,DLconc,noptions_aer,
     $     options_aer,nesp_aer,nbin_aer,ncycle_aer,DLLWC,DLrain,
     $     bin_bound_aer,fixed_density_aer,density_aer,
     $     couples_coag,first_index_coag,second_index_coag,
     $     coefficient_coag,vertical_interface,
     $     DLconc_aer,Wet_Deposition,Wet_Deposition_aer,
     $     pH,saturation_pressure, partition_coefficient,
     $     vaporization_enthalpy, accomodation_coefficient,
     $     surface_tension, saturation_pressure_mass,
     $     saturation_pressure_torr, deliquescence_relative_humidity,
     $     molecular_weight_aer, molecular_diameter_aer,
     $     collision_factor_aer, mass_density_aer,
     $     aerosol_species_interact_loc, isorropia_species_loc,
     $     aec_species_loc, pankow_species_loc,
     $     poa_species_loc, md_species_loc, bc_species_loc,
     $     nesp_cloud_interact,cloud_species_interact)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes one timestep for gas-phase chemistry RADM.
C     Chemical kinetics is solved in each grid cell.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C
C     Nx: number of cells in X direction (longitude).
C     Ny: number of cells in Y direction (latitude).
C     Nz: number of vertical levels.
C     NESP: number of gas species.
C     LWCmin: air liquid water content threshold.
C     TS: initial time (GMT, computed from January 1st, [s]).
C     DLHUMID: 3D specific humidity field at initial time ([%]).
C     DLTEMP: 3D temperature field at initial time ([K]).
C     DLPRESS: 3D pressure field at initial time ([Pa]).
C     DELTA_T: time step ([s]). 
C     NOPTIONS_AER: number of aerosol module options.
C     OPTIONS_AER: 1D list of aerosol module options.
C     NESP_AER: number of aerosol species.
C     NBIN_AER: number of aerosol bins.
C     NCYCLE_AER: number of cycle in aerosol computation.
C     DLLWC: 3D air liquid water content ([fraction]).
C     DLRAIN: 2D field of rain rate at initial time.
C     BIN_BOUND_AER: aerosol diameters at bin bounds.
C     FIXED_DENSITY_AER: fixed aerosol density ([g/m^3]).
C     DENSITY_AER: size variable aerosol density ([g/m^3]).
C     COUPLES_COAG: coagulation couples for each bin.
C     FIRST_INDEX_COAG: first bin index of coagulation couples.
C     SECOND_INDEX_COAG: second bin index of coagulation couples.
C     COEFFICIENT_COAG: coagulation partition coefficient.
C     VERTICAL_INTERFACE: 1D field of vertical interface height ([m]).
C
C     -- INPUT/OUTPUT VARIABLES
C
C     DLCONC: array of 3D gas concentrations ([\mu.g/m^3]). 
C     DLCONC_AER: array of 3D aerosol concentrations ([\mu.g/m^3]). 
C     # Before entry, it is given at initial time of the timestep.
C     # On exit, it is computed at final time of the timestep.
C     
C     -- OUTPUT VARIABLES
C
C     Wet_Deposition: 2D wet fluxes of gaseous species due to in-cloud
C     scavenging ([\mu.g/m^2/s]).
C     Wet_Deposition_aer: 2D wet fluxes of particulate species due to 
C     in-cloud scavenging ([\mu.g/m^2/s]).
C
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2002/02/26: new treatment of sources (Jaouad Boutahar, CEREA).
C     2006/09/29: updated header (Edouard Debry).
C     2010/03/04: SIMPLE_AQUEOUS option included (Youngseob KIM)
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Edouard Debry, CEREA, September 2006.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      INCLUDE 'CONST.INC'
      INCLUDE 'CONST_A.INC'
      INCLUDE 'param.inc'
      INCLUDE 'dynaero.inc'
      INCLUDE 'varg.inc'
      INCLUDE 'vara.inc'

      DOUBLE PRECISION ts,delta_t

      integer nx,ny,nz,nesp

      DOUBLE PRECISION DLconc(NX,NY,NZ,nesp),DLtemp(NX,NY,NZ)
      DOUBLE PRECISION DLhumid(NX,NY,NZ),DLpress(NX,NY,NZ)

      INTEGER Jt,Ji,Jj,Jk,Jsp,i,Jb

      INTEGER nesp_aer,nbin_aer,ncycle_aer
      INTEGER noptions_aer,options_aer(noptions_aer)
      DOUBLE PRECISION DLLWC(NX,NY,NZ),DLrain(Nx,Ny)
      DOUBLE PRECISION bin_bound_aer(nbin_aer + 1)
      DOUBLE PRECISION density_aer(nbin_aer),fixed_density_aer
      INTEGER couples_coag(nbin_aer)
      INTEGER first_index_coag(nbin_aer, 4 * nbin_aer)
      INTEGER second_index_coag(nbin_aer, 4 * nbin_aer)
      DOUBLE PRECISION coefficient_coag(nbin_aer, nbin_aer, nbin_aer)
      DOUBLE PRECISION vertical_interface(NZ + 1)
      DOUBLE PRECISION layer_height(NZ)
      DOUBLE PRECISION DLconc_aer(NX,NY,NZ,nbin_aer,nesp_aer)

      DOUBLE PRECISION Wet_Deposition(NX,NY,nesp)
      DOUBLE PRECISION Wet_Deposition_aer(NX,NY,nbin_aer,nesp_aer)

      INTEGER ICLD,IINCLD
      DOUBLE PRECISION rho_aero(nbin_aer)

      DOUBLE PRECISION lwca(NX,NY,NZ),pH(NX,NY,NZ)
      DOUBLE PRECISION tschem_aer,tfchem_aer,dtchem_aer
      DOUBLE PRECISION ZA(nesp+nbin_aer*nesp_aer)
      DOUBLE PRECISION rain_rate
      DOUBLE PRECISION qscav(nesp+nbin_aer*nesp_aer)
      DOUBLE PRECISION LWCmin

      DOUBLE PRECISION XSF(nbin_aer),MSF(nbin_aer)
      DOUBLE PRECISION DSF(nbin_aer),XBF(nbin_aer+1)
      DOUBLE PRECISION MBF(nbin_aer+1),DBF(nbin_aer+1)
      DOUBLE PRECISION HSF(nbin_aer)

      INTEGER iq(nesp_aer,nbin_aer)

      DOUBLE PRECISION saturation_pressure(nesp_aer)
      DOUBLE PRECISION partition_coefficient(nesp_aer)
      DOUBLE PRECISION vaporization_enthalpy(nesp_aer)
      DOUBLE PRECISION accomodation_coefficient(nesp_aer)
      DOUBLE PRECISION surface_tension(nesp_aer)
      DOUBLE PRECISION saturation_pressure_mass(nesp_aer)
      DOUBLE PRECISION saturation_pressure_torr(nesp_aer)
      DOUBLE PRECISION deliquescence_relative_humidity(nesp_aer)

      DOUBLE PRECISION molecular_weight_aer(nesp_aer)
      DOUBLE PRECISION molecular_diameter_aer(nesp_aer)
      DOUBLE PRECISION collision_factor_aer(nesp_aer)
      DOUBLE PRECISION mass_density_aer(nesp_aer)
      INTEGER aerosol_species_interact_loc(nesp_aer)
      
      INTEGER nesp_isorropia_loc,nesp_aec_loc,nesp_pankow_loc
      INTEGER isorropia_species_loc(nesp_isorropia_loc)
      INTEGER aec_species_loc(nesp_aec_loc)
      INTEGER pankow_species_loc(nesp_pankow_loc)
      INTEGER poa_species_loc,md_species_loc,bc_species_loc
      INTEGER nesp_cloud_interact
      INTEGER cloud_species_interact(nesp_cloud_interact)

      INTEGER i1,i2,j1,j2

      IF (nesp_aer.GT.NEXT) THEN
         STOP 'Number of aerosol species exceeds NEXT in param.inc'
      ENDIF

C     Set parameters
      ICOAG = options_aer(1)
      ICOND = options_aer(2)
      INUCL = options_aer(3)
      ICLD = options_aer(4)
      IINCLD = options_aer(5)
      IKELV = options_aer(6)
      IDENS = options_aer(7)
      ICUT = options_aer(8)
      ISULFCOND = options_aer(9)
      KDSLV = options_aer(10)
      IREDIST = options_aer(11)
      ITERN = options_aer(12)
      ITHRM = options_aer(13)
      IOLIGO = options_aer(14)

      DO Jsp=1,NEXT
         PSATREF(Jsp) = 0.D0
         KPARTREF(Jsp) = 0.D0
         DHVAP(Jsp) = 0.D0
         SIGMA(Jsp) = 0.D0
         STICK(Jsp) = 0.D0
         QSATREF(Jsp) = 0.D0
         TSATREF(Jsp) = 0.D0
         DRH(Jsp) = 0.D0
         EMW(Jsp) = 0.D0
         SIGM(Jsp) = 0.D0
         PARM(Jsp) = 0.D0
         LMD(Jsp) = 0.D0
      ENDDO

      DO Jsp = 1,nesp_aer
         PSATREF(Jsp) = saturation_pressure(Jsp)
         KPARTREF(Jsp) = partition_coefficient(Jsp)
         DHVAP(Jsp) = vaporization_enthalpy(Jsp)
         SIGMA(Jsp) = surface_tension(Jsp)
         STICK(Jsp) = accomodation_coefficient(Jsp)
         QSATREF(Jsp) = saturation_pressure_mass(Jsp)
         TSATREF(Jsp) = saturation_pressure_torr(Jsp)
         DRH(Jsp) = deliquescence_relative_humidity(Jsp)
         EMW(Jsp) = molecular_weight_aer(Jsp)
         SIGM(Jsp) = molecular_diameter_aer(Jsp)
         PARM(Jsp) = collision_factor_aer(Jsp)
         LMD(Jsp) = mass_density_aer(Jsp)
      ENDDO

C     Set pointers
      CALL INITPOINT(nbin_aer,nesp_aer,aerosol_species_interact_loc,iq,
     $     nesp_isorropia_loc,isorropia_species_loc, 
     $     nesp_aec_loc,aec_species_loc,
     $     nesp_pankow_loc,pankow_species_loc,
     $     poa_species_loc,md_species_loc, bc_species_loc,
     $     nesp_cloud_interact,cloud_species_interact)

C     Aerosol density converted in microg / microm^3.
      RHOA = fixed_density_aer * 1.D-09
      DO Jb=1,nbin_aer
         rho_aero(Jb) = density_aer(Jb) * 1.D-09
      ENDDO

C     Aerosol discretization converted in microm.
      DO Jb=1,nbin_aer+1
         DBF(Jb) = bin_bound_aer(Jb) * 1.D06
         MBF(Jb) = RHOA * cst_pi6 * DBF(Jb)**3
         XBF(Jb) = DLOG(MBF(Jb))
      ENDDO
      
      DO Jb=1,nbin_aer
         XSF(Jb) = (XBF(Jb) + XBF(Jb+1))*5.D-01
         MSF(Jb) = DSQRT(MBF(Jb) * MBF(Jb+1))
         DSF(Jb) = DSQRT(DBF(Jb) * DBF(Jb+1))
      ENDDO
                                ! width of each fixed bin
      DO Jb=1,nbin_aer
         HSF(Jb) = XBF(Jb+1)-XBF(Jb)
      END DO


C     Cloud liquid water content (g/m^3)
      DO Jk=1,NZ
         DO Jj=j1,j2
            DO Ji=i1,i2     
               lwca(Ji,Jj,Jk)=DLLWC(Ji,Jj,Jk)*1000.d0*
     &              DLpress(Ji,Jj,Jk)/101325.d0*28.97d0/Pr/
     &              DLtemp(Ji,Jj,Jk)
            ENDDO
         ENDDO
      ENDDO

C     Initialize layer_height
      DO Jk=1,Nz
         layer_height(Jk)= vertical_interface(Jk + 1) -
     $        vertical_interface(Jk)

      ENDDO

C     Initialize pH
      DO Jk=1,NZ
         DO Jj=j1,j2
            DO Ji=i1,i2
               pH(Ji,Jj,Jk) = 0.d0
            ENDDO
         ENDDO
      ENDDO

C     Initialize in-cloud wet fluxes.
      DO Jsp=1,Nesp
         DO Jj=j1,j2
            DO Ji=i1,i2
               Wet_Deposition(Ji,Jj,Jsp) = 0.d0
            ENDDO
         ENDDO
      ENDDO

      DO Jsp=1,Nesp_aer
         DO Jb=1,Nbin_aer
            DO Jj=j1,j2
               DO Ji=i1,i2
                  Wet_Deposition_aer(Ji,Jj,Jb,Jsp) = 0.d0
               ENDDO
            ENDDO
         ENDDO
      ENDDO
      

C     Loop on grid cells.
      DO Jt=1,ncycle_aer
         tschem_aer=ts+(Jt-1)*delta_t/ncycle_aer
         tfchem_aer=tschem_aer+delta_t/ncycle_aer
         dtchem_aer=tfchem_aer-tschem_aer
         DO Jk=1,NZ
            DO Jj=j1,j2
               DO Ji=i1,i2

                  DO Jsp=1,NESP
                     ZA(Jsp) = DLconc(Ji,Jj,Jk,Jsp)
                     IF (ZA(Jsp).LT.0.D0) ZA(Jsp) = 0.D0
                  ENDDO
                  
                  DO Jb=1,nbin_aer
                     DO Jsp=1,nesp_aer
                        i = nesp+(Jsp - 1)*nbin_aer+Jb
                        ZA(i) = DLconc_aer(Ji,Jj,Jk,Jb,Jsp)
                        ZA(i) = DMAX1(ZA(i), TINYM)
                     ENDDO
                  ENDDO

                 
                  IF (lwca(Ji,Jj,Jk).GE.LWCmin) THEN
                     IF (IINCLD.EQ.1) then
                           rain_rate = DLrain(Ji,Jj)
                     ELSE
                           rain_rate = 0.d0
                     ENDIF

                     IF (ICLD.EQ.1) THEN
                        CALL VSRMCHEM(nesp,nesp_aer,Nbin_aer,
     s                       rho_aero,RHOA,DBF,DSF,XBF,ZA,
     s                       DLhumid(Ji,Jj,Jk),DLpress(Ji,Jj,Jk),
     s                       DLtemp(Ji,Jj,Jk),lwca(Ji,Jj,Jk),
     s                       tschem_aer,tfchem_aer,
     s                       rain_rate,pH(Ji,Jj,Jk),qscav)

                        DO Jsp=1,Nesp
                           Wet_Deposition(Ji,Jj,Jsp) =
     $                          Wet_Deposition(Ji,Jj,Jsp) +
     $                          qscav(Jsp) * layer_height(Jk) /
     $                          dtchem_aer
                        ENDDO

                        DO Jsp=1,Nesp_aer
                           DO Jb=1,Nbin_aer
                              Wet_Deposition_aer(Ji,Jj,Jb,Jsp) =
     $                             Wet_Deposition_aer(Ji,Jj,Jb,Jsp) +
     $                             qscav(Nesp + Nbin_aer*(Jsp-1) + Jb) *
     $                             layer_height(Jk) / dtchem_aer
                           ENDDO
                        ENDDO

                     ELSE IF(ICLD.EQ.2) THEN ! Use simple aqueous module.
                        CALL SIMPLE_AQUEOUS_MODULE(nesp,nesp_aer,
     $                       Nbin_aer,
     s                       rho_aero,RHOA,DBF,DSF,XBF,ZA,
     s                       DLhumid(Ji,Jj,Jk),DLpress(Ji,Jj,Jk),
     s                       DLtemp(Ji,Jj,Jk),lwca(Ji,Jj,Jk),
     s                       tschem_aer,tfchem_aer,
     s                       rain_rate,pH(Ji,Jj,Jk),qscav)
                        DO Jsp=1,Nesp
                           Wet_Deposition(Ji,Jj,Jsp) =
     $                          Wet_Deposition(Ji,Jj,Jsp) +
     $                          qscav(Jsp) * layer_height(Jk) /
     $                          dtchem_aer
                        ENDDO

                        DO Jsp=1,Nesp_aer
                           DO Jb=1,Nbin_aer
                              Wet_Deposition_aer(Ji,Jj,Jb,Jsp) =
     $                             Wet_Deposition_aer(Ji,Jj,Jb,Jsp) +
     $                             qscav(Nesp + 
     $                             Nbin_aer*(Jsp-1) + Jb) *
     $                             layer_height(Jk) / dtchem_aer
                           ENDDO
                        ENDDO
                     ELSE
                        CALL AERODYN(nesp,nesp_aer,nbin_aer,
     s                       DLtemp(Ji,Jj,Jk),DLpress(Ji,Jj,Jk),
     s                       DLhumid(Ji,Jj,Jk),tschem_aer,tfchem_aer,
     s                       ZA,couples_coag,
     s                       first_index_coag,second_index_coag,
     s                       coefficient_coag,XSF,MSF,DSF,
     s                       XBF,MBF,DBF,HSF,iq)
                     ENDIF
            
                  ELSE
                     CALL AERODYN(nesp,nesp_aer,nbin_aer,
     s                    DLtemp(Ji,Jj,Jk),DLpress(Ji,Jj,Jk),
     s                    DLhumid(Ji,Jj,Jk),tschem_aer,tfchem_aer,
     s                    ZA,couples_coag,
     s                    first_index_coag,second_index_coag,
     s                    coefficient_coag,XSF,MSF,DSF,
     s                    XBF,MBF,DBF,HSF,iq)
                     
                  ENDIF

                  DO Jb=1,nbin_aer
                     DO Jsp=1,nesp_aer
                        i = nesp+(Jsp - 1)*nbin_aer+Jb
                        DLconc_aer(Ji,Jj,Jk,Jb,Jsp) = ZA(i)
                     ENDDO
                  ENDDO

                  DO Jsp=1,nesp
                     DLconc(Ji,Jj,Jk,Jsp) = ZA(Jsp)
                  ENDDO

               ENDDO            ! loop x.
            ENDDO               ! loop y.
         ENDDO                  ! loop z.	             

         IF (ICLD.GE.1.AND.IINCLD.EQ.1) THEN                  
            call settling(i1,i2,j1,j2,Nx,Ny,Nz,nesp_aer,nbin_aer,
     &           DSF,vertical_interface,DLconc_aer,
     &           lwca,tschem_aer,tfchem_aer)
         ENDIF
      ENDDO

      END
