#include <cmath>
#include <iostream>
#include <string>
using namespace std;

#define assert(condition)						\
  if( (condition) == 0 )						\
    {									\
      cout << "La condition '" << #condition << "' a échoué"		\
	   << " à la ligne " << __LINE__ << endl;			\
      exit( 1 );							\
    }

#define DEBUG_MODE_LEVEL 3

#define SELDONDATA_DEBUG_LEVEL_2

#include "SeldonData.hxx"
using namespace SeldonData;

#include "AtmoData.hxx"
using namespace AtmoData;

#include "Common.cxx"
using namespace Polyphemus;

#include "libRa.hxx"

#include "libRa_Louis.hxx"

#include "libRs.hxx"

#include "libFlux.hxx"

#include<float.h>

#include <map>

bool IsNaN(float);
bool IsInf(float);

bool IsNaN(float x)
{
  return (x != x);
}
bool IsInf(float x)
{
  return (x >= DBL_MAX or x <= -DBL_MAX);
}

template<typename T>
T StringToNumber(const std::string& numberAsString)
{
  T valor;

  std::stringstream stream(numberAsString);
  stream >> valor;
  if( stream.fail() )
    {
      std::runtime_error e(numberAsString);
      throw e;
    }
  return valor;
}

int main (int argc, char** argv)
{
  typedef float real;

  // If incorrect arguments...
  if (argc < 2)
    {
      string msg = "Usage:\n";
      msg += string(" ") + argv[0] + " [configuration file]";
      cout << msg << endl;
      return 1;
    }


  /***************************************************/
  /*               Variable declaration              */
  /***************************************************/

  // Configuration files
  string configuration_file, sec_conf_file, default_name("conf.cfg");

  // Species file
  string species_file;

  // Landuse file.
  string LUC_file;
  string LUC_data;

  // Season index
  int Season = 0;

  // Path directory
  string directory_out;

  // Data Files
  //string Roughness_file, WindVelocity_file,
  string WindVelocity_file,
    FrictionVelocity_file,
    Temperature_file, SurfaceTemperature_file,
    Pressure_file, SurfacePressure_file,
    SurfaceRichardson_file;

  // Domain
  int Nt, Nz, Ny, Nx;

  // Number of LUC categories
  int Nc;

  // Number of seasons
  int Nseason = 5;

  // Output steps
  real Delta_t, Delta_y, Delta_x;
  real t_min, y_min, x_min;
  string levels;

  // Dates
  Date date_beg, date_end;

  // Plan area density
  real lambda_p = 0.75;

  // Buildings' mean height
  real H = 10;

  // Displacement height
  real d;

  // Canyon's mean width
  real w = 6;

  // Recirculation region's width
  real w_r = 0;

  // Canyon's caracteristic lengths
  real z_can_recirculation, z_can_ventilated;

  // Average mixing length in the canopy
  real lc;

  // Surface pressure
  real Ps;

  // Surface temperature
  real Ts;

  // Potential temperature
  real pT;

  // Surface potential temperature
  real pTs;

  // Drag coefficient
  real Cd = 1.2;

  // Friction velocity
  real ustar;

  // Wind velocity at first the vertical node
  real uw;

  // Wind velocity at the top of the canopy (z = h)
  real Utop;

  // Coefficient of exponential wind profile inside canopy
  real WindProfile_coeff = 0;

  // Roughness lengths
  real z0, z0_town, z0_wall, z0_street;

  //
  real lm_lim;

  // Atmospheric first level
  real zref;

  // Empirical constants
  real A = 4;

  // Perfect gas constant for air
  real Rair = 282;

  // Specific heat to constant pressure
  real Cp = 1005;

  // Gravity acceleration
  real g = 9.81;

  // Dynamic viscosity of air
  const real DynamicViscosity = 1.60e-5;	// kg/m/s

  // Particles' diameter
  real ParticleDiameter = 2.5e-6;        // PM2.5

  // Particles' density
  real ParticleDensity = 1400.;           // kg/m^3

  // Richardson number
  real Ri;

  // Sedimentation velocity
  real vs;

  // Option
  string option_Ra, option_save, option_Ri, option_z0, option_diag_z0, option_WindProfile, option_stability;

  // Aerodynamic resistances (temp)
  real Ra_roof_temp = 0;
  real Ra_canyon_sup_temp = 0;
  real Ra_canyon_recirculation_temp = 0;
  real Ra_canyon_inf_recirculation_temp = 0;
  real Ra_street_recirculation_temp = 0;
  real Ra_wall_recirculation_temp = 0;
  real Ra_canyon_ventilated_temp = 0;
  real Ra_canyon_inf_ventilated_temp = 0;
  real Ra_street_ventilated_temp = 0;
  real Ra_wall_ventilated_temp = 0;
  // Surface resistances (temp)
  real Rs_roof_temp, Rs_street_temp, Rs_wall_temp = 0;

  // Dry deposition fluxes over urban surfaces
  real Street_recirculation_Flux_temp;
  real Wall_recirculation_Flux_temp;
  real Street_ventilated_Flux_temp;
  real Wall_ventilated_Flux_temp;
  real Roof_Flux_temp;

  // Some characheristic lengths (canyon's geometry)
  float l_can_ventilated;
  float l_can_recirculation;
  float l_street_ventilated;
  float l_street_recirculation;
  float l_wall_ventilated;
  float l_wall_recirculation;

  // Concentration inside the canyon
  float Ccan_recirculation_temp;
  float Ccan_ventilated_temp;

  /***************************************************/
  /*                Configuration file               */
  /***************************************************/

  parse_argument(argc, argv, configuration_file, sec_conf_file,
		 date_beg, date_end, default_name);

#if DEBUG_MODE_LEVEL > 1
  cout << "\t\tDEBUG: fichier de configuration (1) = " << configuration_file << endl;
  cout << "\t\tDEBUG: fichier de configuration (2) = " << sec_conf_file << endl;
  cout << "\t\tDEBUG: Date de debut = " << date_beg << endl;
  cout << "\t\tDEBUG: Date de fin = " << date_end << endl;
#endif

  cout << "Reading configuration file ..." << endl; cout.flush();

  ConfigStreams configuration(configuration_file);
  if(exists(sec_conf_file))
    configuration.AddFile(sec_conf_file);

  /////////////////////
  //      Domain     //
  /////////////////////
  configuration.SetSection("[domain]");

#if DEBUG_MODE_LEVEL > 2
  cout << "\t\tDEBUG: Section [domain]" << endl;
#endif

  configuration.PeekValue("Nz", ">0", Nz);
  configuration.PeekValue("Ny", ">0", Ny);
  configuration.PeekValue("Nx", ">0", Nx);

#if DEBUG_MODE_LEVEL > 1
  cout << "\t\tDEBUG: Nz = " << Nz << endl
       << "\t\tDEBUG: Ny = " << Ny << endl
       << "\t\tDEBUG: Nx = " << Nx << endl;
#endif

  configuration.PeekValue("Delta_t", ">0", Delta_t);
  configuration.PeekValue("Delta_y", ">0", Delta_y);
  configuration.PeekValue("Delta_x", ">0", Delta_x);

  configuration.PeekValue("y_min", y_min);
  configuration.PeekValue("x_min", x_min);
  configuration.PeekValue("Vertical_levels", levels);

  /////////////////////
  //      Dates      //
  /////////////////////

  string date_meteo_str;
  configuration.PeekValue("Date", date_meteo_str);
  Date date_meteo(date_meteo_str);

#if DEBUG_MODE_LEVEL > 2
  cout << "\t\tDEBUG: Date meteo = " << date_meteo << endl
       << "\t\tDEBUG: Date dep_beg = " << date_beg << endl
       << "\t\tDEBUG: Date dep_end = " << date_end << endl;
#endif

  double difference = date_beg.GetSecondsFrom(date_meteo);
  if (difference < 0)
    throw string("\nthe date you provide in command line ")
      + "should be after the date in the main configuration file.";
  int step = int(difference  / 3600 / Delta_t + 0.5);

#if DEBUG_MODE_LEVEL > 2
  cout << "\t\tDEBUG: difference = " << difference << endl
       << "\t\tDEBUG: step = " << step << endl;
#endif
  
  Nt = compute_Nt(date_beg, date_end, Delta_t);
  t_min = date_beg.GetHour() + real(date_beg.GetMinutes()) / 60.
    + real(date_beg.GetSeconds()) / 3600.;

#if DEBUG_MODE_LEVEL > 0
  cout << "\t\tDEBUG: Nt = " << Nt << endl;
#endif
#if DEBUG_MODE_LEVEL > 2
    cout << "\t\tDEBUG: t_min = " << t_min << endl;
#endif


  ////////////////////
  //      Path      //
  ////////////////////
  configuration.SetSection("[paths]");

#if DEBUG_MODE_LEVEL > 2
  cout << "\t\tDEBUG: Section [paths]" << endl;
#endif

  configuration.PeekValue("Directory_dep", directory_out);

#if DEBUG_MODE_LEVEL > 2
  cout << "\t\tDEBUG: directory_out = " << directory_out << endl;
#endif


  ////////////////////
  //      Data      //
  ////////////////////
  cout << "\tParsing data binaries...";

  configuration.PeekValue("Temperature", Temperature_file);
  configuration.PeekValue("SurfaceTemperature", SurfaceTemperature_file);
  configuration.PeekValue("Pressure", Pressure_file);
  configuration.PeekValue("SurfacePressure", SurfacePressure_file);
  configuration.PeekValue("SurfaceRichardson", SurfaceRichardson_file);
  configuration.PeekValue("WindModule", WindVelocity_file);
  configuration.PeekValue("FrictionVelocity", FrictionVelocity_file);

#if DEBUG_MODE_LEVEL > 2
  cout << endl << "\t\tDEBUG: Temperature_file = " << Temperature_file << endl;
#endif

  cout << "\tdone." << endl;

  ////////////////////
  //       LUC      //
  ////////////////////

  cout << "\tConfiguring Land Use Classification...";

  configuration.SetSection("[LUC]");
  configuration.PeekValue("File", LUC_file);
  Nc = int(file_size(LUC_file)) / sizeof(float) / (Ny * Nx);

  configuration.PeekValue("Data", LUC_data);

#if DEBUG_MODE_LEVEL > 1
  cout << endl << "\t\tDEBUG: Nc = " << Nc << endl;
#endif

  cout << "\tdone." << endl;


  /////////////////////
  //      Urban      //
  // characteristics //
  /////////////////////
  cout << "\tUrban characteristics..." << endl;

  configuration.SetSection("[Urban]");

  vector<string> string_tmp_list;
  //real tmp;
  vector<int> luc_city_index_list;
  int Ncity;
  try
    {
      configuration.Find("LUC_city");
      split(configuration.GetLine(), string_tmp_list);

      for (vector<string>::const_iterator it = string_tmp_list.begin();
	   it != string_tmp_list.end(); ++it)
	luc_city_index_list.push_back(StringToNumber<int>(*it));

      string_tmp_list.clear();

      Ncity = int(luc_city_index_list.size());
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Problem happens when configuring the luc city indexes..." << endl;
      return 1;
    }

#if DEBUG_MODE_LEVEL > 1
  cout << "\t\tDEBUG: Number of city cats = " << Ncity << endl;
  cout << "\t\tDEBUG: LUC city indexes = [";
  for (vector<int>::const_iterator it = luc_city_index_list.begin();
       it != luc_city_index_list.end(); ++it)
    cout << *it << ", ";
  cout << "]" << endl;
#endif

  vector<real> heights_list;
  try
    {
      configuration.Find("Height");
      split(configuration.GetLine(), string_tmp_list);

      for (vector<string>::const_iterator it = string_tmp_list.begin();
	   it != string_tmp_list.end(); ++it)
	heights_list.push_back(StringToNumber<real>(*it));

      string_tmp_list.clear();

      // Check if there is enough height parameters:
      if(int(heights_list.size()) != Ncity)
	throw string("Not enough height parameters were provided !");
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Problem happens when configuring the height parameters..." << endl;
      return 1;
    }
#if DEBUG_MODE_LEVEL > 1
  cout << "\t\tDEBUG: Building heights = [";
  for (vector<real>::const_iterator it = heights_list.begin();
       it != heights_list.end(); ++it)
    cout << *it << ", ";
  cout << "]" << endl;
#endif

  vector<real> road_width_list;
  try
    {
      configuration.Find("Road_width");
      split(configuration.GetLine(), string_tmp_list);

      for (vector<string>::const_iterator it = string_tmp_list.begin();
	   it != string_tmp_list.end(); ++it)
	road_width_list.push_back(StringToNumber<real>(*it));

      string_tmp_list.clear();

      // Check if there is enough height parameters:
      if(int(road_width_list.size()) != Ncity)
	throw string("Not enough road width parameters were provided !");
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Problem happens when configuring the road width parameters..." << endl;
      return 1;
    }

#if DEBUG_MODE_LEVEL > 1
  cout << "\t\tDEBUG: Road width = [";
  for (vector<real>::const_iterator it = road_width_list.begin();
       it != road_width_list.end(); ++it)
    cout << *it << ", ";
  cout << "]" << endl;
#endif

  vector<real> building_density_list;
  try
    {
      configuration.Find("Density");
      split(configuration.GetLine(), string_tmp_list);

      for (vector<string>::const_iterator it = string_tmp_list.begin();
	   it != string_tmp_list.end(); ++it)
	building_density_list.push_back(StringToNumber<real>(*it));

      string_tmp_list.clear();

      // Check if there is enough height parameters:
      if(int(building_density_list.size()) != Ncity)
	throw string("Not enough building density parameters were provided !");
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Problem happens when configuring the building density parameters..." << endl;
      return 1;
    }

#if DEBUG_MODE_LEVEL > 1
  cout << "\t\tDEBUG: Building density = [";
  for (vector<real>::const_iterator it = building_density_list.begin();
       it != building_density_list.end(); ++it)
    cout << *it << ", ";
  cout << "]" << endl;
#endif

  configuration.PeekValue("A", ">0", A);
  configuration.PeekValue("lm_lim", ">0", lm_lim);

#if DEBUG_MODE_LEVEL > 0
  cout << endl 
       << "\t\tDEBUG: A = " << A << endl
       << "\t\tDEBUG: lm_lim = " << lm_lim << endl
    ;
#endif

  cout << "\tdone." << endl;

  /////////////////////
  //    Particles'   //
  // characteristics //
  /////////////////////
  cout << "\tParticles' characteristics..." << endl;

  configuration.SetSection("[Species]");

  try
    {
      configuration.PeekValue("Diameter", ">0", ParticleDiameter);
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Please, reload with proper value of the particles' diameter." << endl;
      return 1;
    }


  try
    {
      // Reads the configuration file associated with the species' data.
      configuration.PeekValue("Density", ">0", ParticleDensity);
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Please, reload with proper value of the particles' density." << endl;
      return 1;
    }

  ParticleDiameter = ParticleDiameter * 1.e-6;

#if DEBUG_MODE_LEVEL > 2
  cout << "\t\tDEBUG: ParticleDiameter = " << ParticleDiameter << endl;
  cout << "\t\tDEBUG: ParticleDensity = " << ParticleDensity << endl;
#endif

  cout << "\tdone." << endl;

  ////////////////////
  //    Options     //
  ////////////////////
  cout << "\tOptions...";

  configuration.SetSection("[Options]");

  try
    {
      configuration.PeekValue("Ra",
			      "canyon_U | masson | roughness",
			      option_Ra);
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Please, reload with proper value of the Ra option." << endl;
      return 1;
    }
  
  try
    {
      configuration.PeekValue("Stability", "yes | no", option_stability);
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Please, reload with proper value of the stability option." << endl;
      return 1;
    }

  try
    {
      configuration.PeekValue("Save", "yes | no", option_save);
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Please, reload with proper value of the save option." << endl;
      return 1;
    }

  try
    {
      configuration.PeekValue("Ri_diag", "yes | no", option_Ri);
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Please, reload with proper value of the Ri option." << endl;
      return 1;
    }

  try
    {
      configuration.PeekValue("z0_diag", "yes | no", option_diag_z0);
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Please, reload with proper value of the z0 diag option." << endl;
      return 1;
    }

  try
    {
      configuration.PeekValue("z0", "Macdonald_1 | Macdonald_2", option_z0);
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Please, reload with proper value of the z0 option." << endl;
      return 1;
    }

  try
    {
      configuration.PeekValue("Wind_profile", "masson | macdonald | value", option_WindProfile);
    }
  catch(string e)
    {
      cout << endl << endl << e << endl << endl;
      cout << "Please, reload with proper value of the Wind profile option." << endl;
      return 1;
    }

  
  if(option_WindProfile == "value")
    configuration.PeekValue("Wind_profile_param", ">0", WindProfile_coeff);

#if DEBUG_MODE_LEVEL > 1
  cout << endl << "\t\tDEBUG: Ra -> " << option_Ra << ", Ri diag -> " << option_Ri
       << ", z0 diag -> " << option_diag_z0 << ", z0 -> " << option_z0
       << ", save -> " << option_save << ", stability -> " << option_stability
       << " and Wind profile coefficient -> " << option_WindProfile << endl;
#endif

  cout << "\tdone." << endl;
  
  cout << "done." << endl;
  
  /***************************************************/
  /*                 Grids allocation                */
  /***************************************************/

  cout << "Memory allocation for data fields..." << endl;

  cout << "\tInput grids...";

  //Input grids
  RegularGrid<real> GridT(t_min, Delta_t, Nt);
  RegularGrid<real> GridZ(Nz);
  RegularGrid<real> GridZ_interf(Nz+1);
  RegularGrid<real> GridY(y_min, Delta_y, Ny);
  RegularGrid<real> GridX(x_min, Delta_x, Nx);
  RegularGrid<real> GridC(Nc);

  // Reads output altitudes.
  FormatText Heights_out;
  Heights_out.Read(levels.c_str(), GridZ_interf);
  // Sets values at nodes.
  for (int k = 0; k < Nz; k++)
    GridZ(k) = ( GridZ_interf(k) + GridZ_interf(k+1) ) / 2.0;

  cout << "\tdone." << endl;
 
  ////////////////////
  //      Data      //
  ////////////////////

  cout << "\tData grids...";

  Data<real, 3> LandUse(GridC, GridY, GridX);
  Data<real, 4> Temperature(GridT, GridZ, GridY, GridX);
  Data<real, 3> SurfaceTemperature(GridT, GridY, GridX);
  Data<real, 4> Pressure(GridT, GridZ, GridY, GridX);
  Data<real, 3> SurfacePressure(GridT, GridY, GridX);
  Data<real, 3> SurfaceRichardson(GridT, GridY, GridX);
  Data<real, 4> WindVelocity(GridT, GridZ, GridY, GridX);
  Data<real, 3> FrictionVelocity(GridT, GridY, GridX);

  // Bulk aerodynamic resistance
  Data<real, 3> Ra(GridT, GridY, GridX);
  // Bulk surface resistance
  Data<real, 3> Rs(GridT, GridY, GridX);

  // Aerodynamic resistance above the canyon
  Data<real, 3> Ra_canyon_sup(GridT, GridY, GridX);
  // Aerodynamic resistance of canyons (recirculation region)
  Data<real, 3> Ra_canyon_recirculation(GridT, GridY, GridX);
  Data<real, 3> Ra_canyon_inf_recirculation(GridT, GridY, GridX);
  // Aerodynamic resistance streets (recirculation region)
  Data<real, 3> Ra_street_recirculation(GridT, GridY, GridX);
  // Aerodynamic resistance walls (recirculation region)
  Data<real, 3> Ra_wall_recirculation(GridT, GridY, GridX);
  // Aerodynamic resistance of canyons (ventilated region)
  Data<real, 3> Ra_canyon_ventilated(GridT, GridY, GridX);
  Data<real, 3> Ra_canyon_inf_ventilated(GridT, GridY, GridX);
  // Aerodynamic resistance of the street (ventilated region)
  Data<real, 3> Ra_street_ventilated(GridT, GridY, GridX);
  // Aerodynamic resistance of walls (ventilated region)
  Data<real, 3> Ra_wall_ventilated(GridT, GridY, GridX);
  // Aerodynamic resistance of roofs
  Data<real, 3> Ra_roof(GridT, GridY, GridX);

  // Surface resistance of streets
  Data<real, 3> Rs_street(GridT, GridY, GridX);
  // Surface resistance of walls
  Data<real, 3> Rs_wall(GridT, GridY, GridX);
  // Surface resistance of roofs
  Data<real, 3> Rs_roof(GridT, GridY, GridX);

  // Sedimentation velocity
  Data<real, 3> SedimentationVelocity(GridT, GridY, GridX);

  // Dry deposition velocity
  Data<real, 3> DepositionVelocity(GridT, GridY, GridX);

  // Fluxes
  Data<real, 3> Street_recirculation_Flux(GridT, GridY, GridX);
  Data<real, 3> Wall_recirculation_Flux(GridT, GridY, GridX);
  Data<real, 3> Street_ventilated_Flux(GridT, GridY, GridX);
  Data<real, 3> Wall_ventilated_Flux(GridT, GridY, GridX);
  Data<real, 3> Roof_Flux(GridT, GridY, GridX);

  // Land data parameterization (Zhang, 2001)
  Array<real, 2> LandRoughnessHeight(Nc, Nseason);
  Array<real, 2> Radius(Nc, Nseason);
  Array<real, 1> Alpha(Nc);
  const real Beta = 2.;
  Array<real, 1> Gamma(Nc);

  cout << "\tdone." << endl;

  cout << "done." << endl;

  /***************************************************/
  /*                    Input data                   */
  /***************************************************/

  cout << "Extracting input data..." << endl;

  ////////////////////
  //    Land Data   //
  ////////////////////

  cout << "\tParsing Land data...";

  // Reads the configuration file associated with the land use categories.
  ConfigStream LUC_param(LUC_data);

  LUC_param.SetSection("[roughness]");
  for (int h = 0; h < Nc; h++)
    for (int s = 0; s < Nseason; s++)
      LUC_param.GetNumber(LandRoughnessHeight(h, s));

  // Reads characteristic radius for all land categories and all five seasons.
  LUC_param.SetSection("[radius]");
  for (int h = 0; h < Nc; h++)
    for (int s = 0; s < Nseason; s++)
      LUC_param.GetNumber(Radius(h, s));

  // Reads Zhang coefficients alpha and gamma.
  LUC_param.SetSection("[zhang_alpha]");
  for (int h = 0; h < Nc; h++)
    LUC_param.GetNumber(Alpha(h));

  LUC_param.SetSection("[zhang_gamma]");
  for (int h = 0; h < Nc; h++)
    LUC_param.GetNumber(Gamma(h));

  cout << "\tdone." << endl;

  ////////////////////
  //   Meteo Data   //
  ////////////////////
  cout << "\tExtracting meteorological fields...";

  FormatBinary<float> InputPolair;

  InputPolair.Read(LUC_file, LandUse);

  try
    {
      InputPolair.ReadSteps(Temperature_file, step, Temperature);
    }
  catch(SeldonData::IOError &e)
    {
      cout << endl << "\t\tErreur: ";
      e.What();
      cout << endl;
      return 1;
    }

  try
    {
      InputPolair.ReadSteps(Pressure_file, step, Pressure);
    }
  catch(SeldonData::IOError &e)
    {
      cout << endl << "\t\tErreur: ";
      e.What();
      cout << endl;
      return 1;
    }

  try
    {
      InputPolair.ReadSteps(SurfaceTemperature_file, step, SurfaceTemperature);
    }
  catch(SeldonData::IOError &e)
    {
      cout << endl << "\t\tErreur: ";
      e.What();
      cout << endl;
      return 1;
    }

  try
    {
      InputPolair.ReadSteps(SurfacePressure_file, step, SurfacePressure);
    }
  catch(SeldonData::IOError &e)
    {
      cout << "\t\tErreur: ";
      e.What();
      cout << endl;
      return 1;
    }

  try
    {
      InputPolair.ReadSteps(SurfaceRichardson_file, step, SurfaceRichardson);
    }
  catch(SeldonData::IOError &e)
    {
      cout << "\t\tErreur: ";
      e.What();
      cout << endl;
      return 1;
    }

  try
    {
      InputPolair.ReadSteps(WindVelocity_file, step, WindVelocity);
    }
  catch(SeldonData::IOError &e)
    {
      cout << "\t\tErreur: ";
      e.What();
      cout << endl;
      return 1;
    }

  try
    {
      InputPolair.ReadSteps(FrictionVelocity_file, step, FrictionVelocity);
    }
  catch(SeldonData::IOError &e)
    {
      cout << "\t\tErreur: ";
      e.What();
      cout << endl;
      return 1;
    }

  cout << "\tdone." << endl;

  cout << "done." << endl;

  ////////////////////
  //  Computed data //
  ////////////////////

  cout << "Computing deposition velocity..." << endl;

  zref = GridZ(0);
#if DEBUG_MODE_LEVEL > 1
  cout << "\t\tDEBUG: zref = " << zref << endl;
#endif

  real AirDensity = 1.0;

  Date current_date = date_beg;
  int Time_in_days = 0;
  int Old_day = -1;
#if DEBUG_MODE_LEVEL > 1
  int OldSeason = 0;
#endif

  cout << endl;

  std::vector<int>::iterator iter;
  size_t index_luc_city;

  for (int k = 0; k < Nt; k++)
    {
      ////////////////////
      // Compute season //
      ////////////////////
#if DEBUG_MODE_LEVEL > 1
      OldSeason = Season;
#endif

      Time_in_days =  current_date.GetDayNumber();
      if(Time_in_days != Old_day)
	cout << "\tDay " << current_date.GetDate() << endl;
      
      if (Time_in_days <= 59 || Time_in_days > 303)
	// Season = 1 (winter: november, december, january, february)
	Season = 0;
      else if (Time_in_days <= 120)
	// Season = 2 (spring: march, april)
	Season = 1;
      else if (Time_in_days <= 242)
	// Season = 3 (summer: may, june, july, august)
	Season = 2;
      else
	// Season = 4 (fall: september, october)
	Season = 3;

#if DEBUG_MODE_LEVEL > 1
      if(Season != OldSeason)
	cout << "\t\tDEBUG: Season = " << Season << endl;
#endif

      for (int j = 0; j < Ny; j++)
	{
	  for (int i = 0; i < Nx; i++)
	    {
	      // Surface pressure
	      Ps = SurfacePressure(k, j, i);

	      // Surface temperature
	      Ts = SurfaceTemperature(k, j, i);

	      // Wind module
	      uw = WindVelocity(k, 0, j, i);

	      // Richardson number
	      if(option_Ri == "no")
		{
		  Ri = SurfaceRichardson(k, j, i);
		}
	      else
		{
		  // Surface potential temperature
		  pTs = ComputeSurfacePotentialTemp(Ts, Ps, Rair, Cp);
		  // Potential temperature
		  pT = ComputePotentialTemp(Temperature(k, 0, j, i),
					    Pressure(k, 0, j, i), Rair, Cp);
		  Ri = ComputeRichardson(pTs, pT, g, zref, uw);
		}

	      // Sedimentation velocity
	      vs = ComputeSedimentationVelocity(DynamicViscosity,
						Ps, Ts,	ParticleDiameter, 
						ParticleDensity);

#if DEBUG_MODE_LEVEL > 0
	      if(vs < 0 or vs != vs)
		{
		  cout << "\t\tDEBUG: vs(" << k << ", " << j
		       << ", " << i << ") = " << vs << endl;
		  return 1;
		}
#endif

	      for (int h = 0; h < Nc; h++)
		{
		  if(LandUse(h, j, i) != 0)
		    {
		  // Bulk aerodynamic resistance (temp)
		  real Ra_temp = 0;
		  // Bulk surface resistance (temp)
		  real Rs_temp = 0;

		  // Dry deposition velocity (temp)
		  real vd_temp = 0;

		  ///////////////////////////
		  //    Roughness height   //
		  ///////////////////////////

		  iter = std::find(luc_city_index_list.begin(), 
				   luc_city_index_list.end(), h+1);

		  index_luc_city = std::distance(luc_city_index_list.begin(), iter);

		  // If urban cells and diagnoze roughness height:
		  if(index_luc_city != luc_city_index_list.size() && option_diag_z0 == "yes")
		    {
		      if(option_z0 == "Macdonald_1")
			{
			  // Displacement height
			  d = ComputeD(A, building_density_list[index_luc_city],
				       heights_list[index_luc_city]);
			  // Roughness height
			  z0 = ComputeZ0(building_density_list[index_luc_city], heights_list[index_luc_city], d, Cd);
			}
		      else if(option_z0 == "Macdonald_2")
			{
			  //z0 = computeZ0(lambda_p, h);
			  z0 = computeZ0(building_density_list[index_luc_city], heights_list[index_luc_city]);
			}
		    }
		  // Else if non urban cells, or non-diagnose roughness height
		  else // option_diag_z0 == "no"
		    {
		      // Roughness height
		      z0 = LandRoughnessHeight(h, Season);
		    }

		  // Urban LUC AND new parametrization
		  if(index_luc_city != luc_city_index_list.size() 
		     && option_Ra != "roughness")
		    {

#if DEBUG_MODE_LEVEL > 3
		      cout << "\t\tDEBUG: LUC(" << k << ", " << j
			   << ", " << i << ") = Urban" << endl;
#endif
		      lambda_p = building_density_list[index_luc_city];
		      H = heights_list[index_luc_city];
		      w = road_width_list[index_luc_city];
		      
		      if(option_WindProfile == "masson")
			{
			  WindProfile_coeff = 0.5 * H / w;
			}
		      else if(option_WindProfile == "macdonald")
			{
			  WindProfile_coeff = 9.6 * lambda_p;
			}

		      // Definition of the characteristic canyon's lengths
		      w_r = 3. * H;

#if DEBUG_MODE_LEVEL > 3
		      cout << endl << "\t\tDEBUG: w_r = " << w_r << endl;
#endif

		      l_can_ventilated = w - min( w_r / 2., w );
		      l_can_recirculation = min( w_r / 2., w );
		      l_street_ventilated = w - min( w_r, w );
		      l_street_recirculation = min( w_r, w );
		      if(w < w_r / 2.)
			{
			  l_wall_ventilated = 0.;
			  l_wall_recirculation = 2. * H;
			}
		      else if(w > w_r / 2. && w < w_r)
			{
			  l_wall_ventilated = 2. * H * ( w / w_r - 0.5 );
			  l_wall_recirculation = 2. * H * ( 1.5 - w / w_r );
			}
		      else if(w > w_r)
			{
			  l_wall_ventilated = H;
			  l_wall_recirculation = H;
			}

		      // Displacement height
		      d = ComputeD(A, lambda_p, H);

		      // Canopy mix length
		      lc = ComputeLc(H, d);

		      z0_town = z0;
		      z0_wall = 0.001;
		      z0_street = 0.01;

		      ///////////////////////////
		      //        Dynamic        //
		      ///////////////////////////

		      // Ustar (urban area)
		      if(option_stability == "yes")
			ustar = ComputeUstar_AboveCanopy(zref, z0_town, d, Ri, uw);
		      else
			ustar = ComputeUstar_AboveCanopy(zref, z0_town, d, uw);

		      // Utop included canyon integration
		      Utop = ComputeUtop_CanyonIntegration(w, H, w_r)
			* ComputeUtop(zref, H, d, ustar, uw);


		      ////////////////////////////////////////////////////
		      //       Computation of surface resistances       //
		      ////////////////////////////////////////////////////

		      // Surface resistance of streets
		      Rs_street_temp =
			ComputeSurfaceResistance(DynamicViscosity,
						 Ps, Ts, ParticleDiameter, 
						 ParticleDensity, AirDensity,
						 ustar, Radius(h, Season),
						 Alpha(h), Beta, Gamma(h));

		      // Surface resistance of walls
		      Rs_wall_temp =
			ComputeSurfaceResistance(DynamicViscosity,
						 Ps, Ts, ParticleDiameter, 
						 ParticleDensity, AirDensity,
						 ustar, Radius(h, Season),
						 Alpha(h), Beta, Gamma(h));

		      // Surface resistance of roofs
		      Rs_roof_temp =
			ComputeSurfaceResistance(DynamicViscosity,
						 Ps, Ts, ParticleDiameter, 
						 ParticleDensity, AirDensity,
						 ustar, Radius(h, Season),
						 Alpha(h), Beta, Gamma(h));

#if DEBUG_MODE_LEVEL > 0
			  if(Rs_street_temp < 0 or
			     IsNaN(Rs_street_temp) or
			     IsInf(Rs_street_temp) )
			    {
			      cout << "\t\tDEBUG: Rs_street(" << k << ", " << j
				   << ", " << i << ") = " << Rs_street_temp << endl;
			      return 1;
			    }
			  if(Rs_wall_temp < 0 or
			     IsNaN(Rs_wall_temp) or
			     IsInf(Rs_wall_temp) )
			    {
			      cout << "\t\tDEBUG: Rs_wall(" << k << ", " << j
				   << ", " << i << ") = " << Rs_wall_temp << endl;
			      return 1;
			    }
			  if(Rs_roof_temp < 0 or
			     IsNaN(Rs_roof_temp) or
			     IsInf(Rs_roof_temp) )
			    {
			      cout << "\t\tDEBUG: Rs_roof(" << k << ", " << j
				   << ", " << i << ") = " << Rs_roof_temp << endl;
			      return 1;
			    }
#endif

		      ////////////////////////////////////////////////////
		      //      Computation of aerodynamic resistances    //
		      //                    (roofs)                     //
		      ////////////////////////////////////////////////////

		      // Assuming turbulence of the canopy prevails over ones of the roof...
		      Ra_roof_temp =
			ComputeRcanSup(zref, H, d, ustar);

		      // Aerodynamical resistance above the canyon [int(zref, h)]
		      Ra_canyon_sup_temp = ComputeRcanSup(zref, H, d, ustar);


		      ////////////////////////////////////////////////////
		      //      Computation of aerodynamic resistances    //
		      //           (in the recirculation region)        //
		      ////////////////////////////////////////////////////

		      // Characteristic canyon length (recirculation region)
		      if( option_Ra == "canyon_U" )
			{

			  ////////////////////////////////////////////////////
			  //      Computation of aerodynamic resistances    //
			  //           (in the recirculation region)        //
			  ////////////////////////////////////////////////////

			  // Characteristic canyon length (recirculation region)
			  z_can_recirculation = ComputeZcan(l_can_recirculation, H);

			  // Resistance of the canyon
			  Ra_canyon_inf_recirculation_temp =
			    ComputeRcanInf_recirculation_exp_profile(lc, z_can_recirculation, H,
								     Utop, WindProfile_coeff);

			  Ra_canyon_recirculation_temp =
			    Ra_canyon_inf_recirculation_temp + Ra_canyon_sup_temp;

			  // Resistance of the street (through canyon)
			  Ra_street_recirculation_temp =
			    ComputeRsurface_recirculation_exp_profile(lc, z_can_recirculation, z0_street,
								      H, Utop, WindProfile_coeff, lm_lim);

			  // Resistance of the wall (through canyon)
			  Ra_wall_recirculation_temp =
			    ComputeRsurface_recirculation_exp_profile(lc, z_can_recirculation, z0_wall,
								      H, Utop, WindProfile_coeff, lm_lim);

			  ////////////////////////////////////////////////////
			  //      Computation of aerodynamic resistances    //
			  //            (in the ventilated region)          //
			  ////////////////////////////////////////////////////

			  // If ventilated region exists...
			  if(w > w_r / 2.)
			    {
			      // Characteristic canyon length (ventilated region)
			      z_can_ventilated = ComputeZcan(l_can_ventilated, H);

			      // Aerodynamic resistance of the canyon
			      //  outside the recirculation region
			      Ra_canyon_inf_ventilated_temp =
				ComputeRcanInf_ventilated_exp_profile(z_can_ventilated, H,
								      Utop, WindProfile_coeff);

			      // Aerodynamic resistance of streets
			      //  outside the recirculation region
			      Ra_street_ventilated_temp =
				ComputeRsurface_ventilated_exp_profile(lc, z_can_ventilated, z0_street, H,
									  Utop, WindProfile_coeff, lm_lim);

			      // Aerodynamic resistance of walls
			      //  outside the recirculation region
			      Ra_wall_ventilated_temp =
				ComputeRsurface_ventilated_exp_profile(lc, z_can_ventilated, z0_street, H,
									  Utop, WindProfile_coeff, lm_lim);
			    }
			  // Narrow street canyon (no ventilated region)
			  else
			    {
			      // Aerodynamic resistance of the canyon
			      //  outside the recirculation region
			      Ra_canyon_ventilated_temp = 0;

			      // Aerodynamic resistance of streets
			      //  outside the recirculation region
			      Ra_street_ventilated_temp = 0;

			      // Aerodynamic resistance of walls
			      //  outside the recirculation region
			      Ra_wall_ventilated_temp = 0;
			    }
			} // end of option_Ra = canyon_U

#if DEBUG_MODE_LEVEL > 0
		      if(Ra_canyon_recirculation_temp < 0 or
			 IsNaN(Ra_canyon_recirculation_temp) or
			 IsInf(Ra_canyon_recirculation_temp) )
			{
			  cout << "\t\tDEBUG: R_can_recirculation(" << k << ", " << j
			       << ", " << i << ") = " << Ra_canyon_recirculation_temp << endl;
			  return 1;
			}
		      if(Ra_street_recirculation_temp < 0 or
			 IsNaN(Ra_street_recirculation_temp) or
			 IsInf(Ra_street_recirculation_temp) )
			{
			  cout << "\t\tDEBUG: R_street_recirculation(" << k << ", " << j
			       << ", " << i << ") = " << Ra_street_recirculation_temp << endl;
			  return 1;
			}
		      if(Ra_wall_recirculation_temp < 0 or
			 IsNaN(Ra_wall_recirculation_temp) or
			 IsInf(Ra_wall_recirculation_temp) )
			{
			  cout << "\t\tDEBUG: R_wall_recirculation(" << k << ", " << j
			       << ", " << i << ") = " << Ra_wall_recirculation_temp << endl;
			  return 1;
			}
		      if(Ra_canyon_ventilated_temp < 0 or
			 IsNaN(Ra_canyon_ventilated_temp) or
			 IsInf(Ra_canyon_ventilated_temp) )
			{
			  cout << "\t\tDEBUG: R_can_ventilated(" << k << ", " << j
			       << ", " << i << ") = " << Ra_canyon_ventilated_temp << endl;
			  return 1;
			}
		      if(Ra_street_ventilated_temp < 0 or
			 IsNaN(Ra_street_ventilated_temp) or
			 IsInf(Ra_street_ventilated_temp) )
			{
			  cout << "\t\tDEBUG: R_street_ventilated(" << k << ", " << j
			       << ", " << i << ") = " << Ra_street_ventilated_temp << endl;
			  return 1;
			}
		      if(Ra_wall_ventilated_temp < 0 or
			 IsNaN(Ra_wall_ventilated_temp) or
			 IsInf(Ra_wall_ventilated_temp) )
			{
			  cout << "\t\tDEBUG: R_wall_ventilated(" << k << ", " << j
			       << ", " << i << ") = " << Ra_wall_ventilated_temp << endl;
			  return 1;
			}
#endif
		      /*******************************************************************/


		      ////////////////////////////////////////////////////
		      //              Computation of fluxes             //
		      //          (in the recirculated region)          //
		      ////////////////////////////////////////////////////

		      Street_recirculation_Flux_temp =
			ComputeStreetFlux(l_can_recirculation, l_wall_recirculation,
					  l_street_recirculation, vs,
					  Ra_canyon_recirculation_temp,
					  Ra_street_recirculation_temp,
					  Rs_street_temp,
					  Ra_wall_recirculation_temp,
					  Rs_wall_temp);

		      Wall_recirculation_Flux_temp = 
			ComputeWallFlux(l_can_recirculation, l_wall_recirculation,
					l_street_recirculation, vs,
					Ra_canyon_recirculation_temp,
					Ra_street_recirculation_temp,
					Rs_street_temp,
					Ra_wall_recirculation_temp,
					Rs_wall_temp);

		      Ccan_recirculation_temp = ComputeCcan(l_can_recirculation, l_wall_recirculation,
							    l_street_recirculation, vs,
							    Ra_canyon_recirculation_temp,
							    Ra_street_recirculation_temp,
							    Rs_street_temp,
							    Ra_wall_recirculation_temp,
							    Rs_wall_temp);

		      ////////////////////////////////////////////////////
		      //              Computation of fluxes             //
		      //            (in the ventilated region)          //
		      ////////////////////////////////////////////////////

		      if(w > w_r / 2.) // If ventilated region exists...
			{

			  Street_ventilated_Flux_temp = 
			    ComputeStreetFlux(l_can_ventilated, l_wall_ventilated,
					      l_street_ventilated, vs,
					      Ra_canyon_ventilated_temp,
					      Ra_street_ventilated_temp,
					      Rs_street_temp,
					      Ra_wall_ventilated_temp,
					      Rs_wall_temp);

			  Wall_ventilated_Flux_temp = 
			    ComputeWallFlux(l_can_ventilated, l_wall_ventilated,
					    l_street_ventilated, vs,
					    Ra_canyon_ventilated_temp,
					    Ra_street_ventilated_temp,
					    Rs_street_temp,
					    Ra_wall_ventilated_temp,
					    Rs_wall_temp);

			  Ccan_ventilated_temp = ComputeCcan(l_can_ventilated, l_wall_ventilated,
							     l_street_ventilated, vs,
							     Ra_canyon_ventilated_temp,
							     Ra_street_ventilated_temp,
							     Rs_street_temp,
							     Ra_wall_ventilated_temp,
							     Rs_wall_temp);
			}
		      else // otherwise, fluxes are equal to zero.
			{
			  Wall_ventilated_Flux_temp = Street_ventilated_Flux_temp = 0;
			}

		      ////////////////////////////////////////////////////
		      //           Computation of roof flux             //
		      ////////////////////////////////////////////////////

		      Roof_Flux_temp = ComputeRoofFlux(vs, Ra_roof_temp,
						       Rs_roof_temp);

#if DEBUG_MODE_LEVEL > 0

		      if(Street_recirculation_Flux_temp < 0 or
			 IsNaN(Street_recirculation_Flux_temp) or
			 IsInf(Street_recirculation_Flux_temp) )
			{
			  cout << "\t\tDEBUG: Street_recirculation_Flux(" << k << ", " << j
			       << ", " << i << ") = " << Street_recirculation_Flux_temp << endl;
			  cout << "\t\tDEBUG: Ccan (" << k << ", " << j
			       << ", " << i << ") = " << Ccan_recirculation_temp << endl;
			  return 1;
			}
		      if(Wall_recirculation_Flux_temp < 0 or
			 IsNaN(Wall_recirculation_Flux_temp) or
			 IsInf(Wall_recirculation_Flux_temp) )
			{
			  cout << "\t\tDEBUG: Wall_recirculation_Flux(" << k << ", " << j
			       << ", " << i << ") = " << Wall_recirculation_Flux_temp << endl;
			  cout << "\t\tDEBUG: Ccan (" << k << ", " << j
			       << ", " << i << ") = " << Ccan_recirculation_temp << endl;
			  return 1;
			}
		      if(Street_ventilated_Flux_temp < 0 or
			 IsNaN(Street_ventilated_Flux_temp) or
			 IsInf(Street_ventilated_Flux_temp) )
			{
			  cout << "\t\tDEBUG: Street_ventilated_Flux(" << k << ", " << j
			       << ", " << i << ") = " << Street_ventilated_Flux_temp << endl;
			  cout << "\t\tDEBUG: Ccan (" << k << ", " << j
			       << ", " << i << ") = " << Ccan_ventilated_temp << endl;
			  return 1;
			}
		      if(Wall_ventilated_Flux_temp < 0 or
			 IsNaN(Wall_ventilated_Flux_temp) or
			 IsInf(Wall_ventilated_Flux_temp) )
			{
			  cout << "\t\tDEBUG: Wall_ventilated_Flux(" << k << ", " << j
			       << ", " << i << ") = " << Wall_ventilated_Flux_temp << endl;
			  cout << "\t\tDEBUG: Ccan (" << k << ", " << j
			       << ", " << i << ") = " << Ccan_ventilated_temp << endl;
			  cout << "\t\tDEBUG: u(h) (" << k << ", " << j
			       << ", " << i << ") = " << Utop << endl;
			  cout << "\t\tDEBUG: u(zref) (" << k << ", " << j
			       << ", " << i << ") = " << uw << endl;
			  cout << "\t\tDEBUG: u* (" << k << ", " << j
			       << ", " << i << ") = " << ustar << endl;
			  cout << "\t\tDEBUG: Ra_can_ventilated(" << k << ", " << j
			       << ", " << i << ") = " << Ra_canyon_ventilated_temp << endl;
			  return 1;
			}
		      if(Roof_Flux_temp < 0 or
			 IsNaN(Roof_Flux_temp) or
			 IsInf(Roof_Flux_temp) )
			{
			  cout << "\t\tDEBUG: Roof_Flux(" << k << ", " << j
			       << ", " << i << ") = " << Roof_Flux_temp << endl;
			  return 1;
			}
#endif


		      /*******************************************************************/


		      ////////////////////////////////////////////////////
		      //    Computation of the visible dry deposition   //
		      //           velocity for urban land-use          //
		      ////////////////////////////////////////////////////

		      vd_temp = ComputeFluxEquiv(lambda_p, H, w,
						 Street_ventilated_Flux_temp,
						 Wall_ventilated_Flux_temp,
						 Street_recirculation_Flux_temp,
						 Wall_recirculation_Flux_temp,
						 Roof_Flux_temp);

#if DEBUG_MODE_LEVEL > 3
		      cout << "\t\tDEBUG: vd_urban(" << k << ", " << j
			   << ", " << i << ") = " << vd_temp << endl;
#endif

#if DEBUG_MODE_LEVEL > 0
		      if(vd_temp < 0 or
			 IsNaN(vd_temp) or
			 IsInf(vd_temp) )
			{
			  cout << "\t\tDEBUG: vd_urban(" << k << ", " << j
			       << ", " << i << ") = " << vd_temp << endl;
			  return 1;
			}
#endif

		      if( option_save == "yes")
			{
			  // Sedimentation velocity
			  SedimentationVelocity(k, j, i) = vs;

			  // Aerodynamic resistances (over urban areas)
			  Ra_roof(k, j, i) = Ra_roof_temp;
			  Ra_canyon_recirculation(k, j, i) = Ra_canyon_recirculation_temp;
			  Ra_canyon_ventilated(k, j, i) = Ra_canyon_ventilated_temp;
			  Ra_canyon_sup(k, j, i) = Ra_canyon_sup_temp;
			  Ra_canyon_inf_recirculation(k, j, i) = Ra_canyon_inf_recirculation_temp;
			  Ra_canyon_inf_ventilated(k, j, i) = Ra_canyon_inf_ventilated_temp;
			  Ra_street_recirculation(k, j, i) =
			    Ra_street_recirculation_temp;
			  Ra_street_ventilated(k, j, i) = Ra_street_ventilated_temp;
			  Ra_wall_recirculation(k, j, i) =
			    Ra_wall_recirculation_temp;
			  Ra_wall_ventilated(k, j, i) = Ra_wall_ventilated_temp;

			  // Surface resistances
			  Rs_street(k, j, i) = Rs_street_temp;
			  Rs_wall(k, j, i) = Rs_wall_temp;
			  Rs_roof(k, j, i) = Rs_roof_temp;

			  // Fluxes
			  Street_recirculation_Flux(k, j, i) =
			    Street_recirculation_Flux_temp;
			  Wall_recirculation_Flux(k, j, i) =
			    Wall_recirculation_Flux_temp;
			  Street_ventilated_Flux(k, j, i) = Street_ventilated_Flux_temp;
			  Wall_ventilated_Flux(k, j, i) = Wall_ventilated_Flux_temp;
			  Roof_Flux(k, j, i) = Roof_Flux_temp;
			} // End of option_save

		      /////////////////////////////////////////
		      //               Flush                 //
		      /////////////////////////////////////////

		      // Aerodynamic resistances (temp)
		      Ra_roof_temp = 0;
		      Ra_canyon_recirculation_temp = 0;
		      Ra_canyon_ventilated_temp = 0;
		      Ra_canyon_sup_temp = 0;
		      Ra_canyon_inf_recirculation_temp = 0;
		      Ra_canyon_inf_ventilated_temp = 0;
		      Ra_street_recirculation_temp = 0;
		      Ra_wall_recirculation_temp = 0;
		      Ra_street_ventilated_temp = 0;
		      Ra_wall_ventilated_temp = 0;
		      // Surface resistances (temp)
		      Rs_roof_temp = 0;
		      Rs_street_temp = 0;
		      Rs_wall_temp = 0;
		      // Fluxes (temp)
		      Street_recirculation_Flux_temp = 0;
		      Wall_recirculation_Flux_temp = 0;
		      Street_ventilated_Flux_temp = 0;
		      Wall_ventilated_Flux_temp = 0;
		      Roof_Flux_temp = 0;

		    } // end of urban parametrization
		  
		  // Non Urban OR roughness parametrization
		  else
		    {

#if DEBUG_MODE_LEVEL > 3
		      cout << "\t\tDEBUG: LUC(" << k << ", " << j
			   << ", " << i << ") = Non Urban" << endl;
#endif

		      ///////////////////////////
		      //         Ustar         //
		      ///////////////////////////
		      // Urban (neutral condition + displacement heigh d = 0.7*height) in roughness approach
		      if(index_luc_city != luc_city_index_list.size())
			{
			  ustar = ComputeUstar_AboveCanopy(zref, z0, 0.7*H, Ri, uw);
			}
		      // Non urban
		      else
			{
			  ustar = ComputeUstar(zref, z0, Ri, uw);
			}

		      // Aerodynamic resistance (from Louis' formula)
		      Ra_temp = ComputeRaLouis(zref, z0, Ri, uw);

		      // Surface resistance
		      Rs_temp =
			ComputeSurfaceResistance(DynamicViscosity,
						 Ps, Ts, ParticleDiameter, 
						 ParticleDensity, AirDensity,
						 ustar, Radius(h, Season),
						 Alpha(h), Beta, Gamma(h));

		      vd_temp = ComputeRoughnessFlux(vs, Ra_temp, Rs_temp);

#if DEBUG_MODE_LEVEL > 0
		      if(vd_temp < 0 or
			 IsNaN(vd_temp) or
			 IsInf(vd_temp) )
			{
			  cout << "\t\tDEBUG: vd_roughness(" << k << ", " << j
			       << ", " << i << ") = " << vd_temp << endl;
			  return 1;
			}
#endif

		      if( option_save == "yes")
			{
			  // Aggregation over LUCs
			  Ra(k, j, i) += LandUse(h, j, i) * Ra_temp;
			  Rs(k, j, i) += LandUse(h, j, i) * Rs_temp;
			}

		    } // End of non urban (or roughness) parameterization

		  // Compute dry deposition velocity
		  DepositionVelocity(k, j, i) += LandUse(h, j, i) * vd_temp;
		  // Flush
		  vd_temp = 0;

		    } // End of LandUse(h, j, i) != 0
		} // Nc
	    } // Nx
	} // Ny

      current_date.AddMinutes(int(Delta_t*60));
      Old_day = Time_in_days;
      cout.flush();

    } // Nt


  cout << "done." << endl;


  /***************************************************/
  /*                  Writing data                   */
  /***************************************************/

  cout << "Writing data..." << endl;

  FormatBinary<float> OutputPolair;

  string file_out;
  file_out = directory_out + "DepositionVelocity_vd.bin";

#if DEBUG_MODE_LEVEL > 2
  cout << "\t\tDEBUG: file_out = " << file_out << endl;
#endif

  OutputPolair.Append(DepositionVelocity, file_out);
  cout << endl << "\tDepositionVelocity_vd.bin written." << endl;

  if( option_save == "yes")
    {
      // Sedimentation velocity
      OutputPolair.Append(SedimentationVelocity, directory_out +
			  "SedimentationVelocity_vs.bin");
      cout << endl << "\tSedimentationVelocity_vs.bin written." << endl;

      if( option_Ra == "roughness" )
	{
	  // Bulk aerodynamic resistance
	  OutputPolair.Append(Ra, directory_out + "Ra_bulk.bin");
	  cout << endl << "\tRa.bin written." << endl;

	  // Bulk surface resistance
	  OutputPolair.Append(Rs, directory_out + "Rs_bulk.bin");
	  cout << endl << "\tRs.bin written." << endl;
	}
      else if( option_Ra != "roughness" )
	{
	  // Aerodynamic resistances
	  OutputPolair.Append(Ra_roof, directory_out + "Ra_roof.bin");
	  cout << "\tRa_roof.bin written." << endl;

	  OutputPolair.Append(Ra_canyon_recirculation, directory_out
			      + "Ra_canyon-recirculation.bin");
	  cout << "\tRa_canyon-recirculation.bin written." << endl;

	  OutputPolair.Append(Ra_street_recirculation,
			      directory_out + "Ra_street-recirculation.bin");
	  cout << "\tRa_street-recirculation.bin written." << endl;

	  OutputPolair.Append(Ra_wall_recirculation,
			      directory_out + "Ra_wall-recirculation.bin");
	  cout << "\tRa_wall-recirculation.bin written." << endl;

	  OutputPolair.Append(Ra_canyon_ventilated, directory_out
			      + "Ra_canyon-ventilated.bin");
	  cout << "\tRa_canyon-ventilated.bin written." << endl;

	  OutputPolair.Append(Ra_street_ventilated,
			      directory_out + "Ra_street-ventilated.bin");
	  cout << "\tRa_street-ventilated.bin written." << endl;

	  OutputPolair.Append(Ra_wall_ventilated,
			      directory_out + "Ra_wall-ventilated.bin");
	  cout << "\tRa_wall-ventilated.bin written." << endl;

	  // Surface resistances
	  OutputPolair.Append(Rs_street, directory_out + "Rs_street.bin");
	  cout << "\tRs_street.bin written." << endl;
	  OutputPolair.Append(Rs_wall, directory_out + "Rs_wall.bin");
	  cout << "\tRs_wall.bin written." << endl;
	  OutputPolair.Append(Rs_roof, directory_out + "Rs_roof.bin");
	  cout << "\tRs_roof.bin written." << endl;

	  // Fluxes
	  OutputPolair.Append(Street_recirculation_Flux,
			      directory_out + "Flux_street-recirculation.bin");
	  cout << "\tFlux_street-recirculation.bin written." << endl;
	  OutputPolair.Append(Wall_recirculation_Flux,
			      directory_out + "Flux_wall-recirculation.bin");
	  cout << "\tFlux_wall-recirculation.bin written." << endl;
	  OutputPolair.Append(Street_ventilated_Flux,
			      directory_out + "Flux_street-ventilated.bin");
	  cout << "\tFlux_street-ventilated.bin written." << endl;
	  OutputPolair.Append(Wall_ventilated_Flux,
			      directory_out + "Flux_wall-ventilated.bin");
	  cout << "\tFlux_wall-ventilated.bin written." << endl;
	  OutputPolair.Append(Roof_Flux,
			      directory_out + "Flux_roof.bin");
	  cout << "\tFlux_roof.bin written." << endl;

	}
    }


  cout << "done." << endl;

  return 0;
}
